package com.battlelancer.seriesguide.ui;

import android.app.SearchManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import androidx.appcompat.app.ActionBar;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;
import androidx.viewpager.widget.ViewPager;
import butterknife.ButterKnife;
import com.battlelancer.seriesguide.R;
import com.battlelancer.seriesguide.adapters.TabStripAdapter;
import com.battlelancer.seriesguide.provider.SgRoomDatabase;
import com.battlelancer.seriesguide.provider.SgShow2Helper;
import com.battlelancer.seriesguide.ui.lists.ManageListsDialogFragment;
import com.battlelancer.seriesguide.ui.overview.OverviewFragment;
import com.battlelancer.seriesguide.ui.overview.SeasonsFragment;
import com.battlelancer.seriesguide.ui.overview.ShowFragment;
import com.battlelancer.seriesguide.ui.search.EpisodeSearchFragment;
import com.battlelancer.seriesguide.ui.shows.RemoveShowDialogFragment;
import com.battlelancer.seriesguide.ui.shows.ShowTools2;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

/**
 * Hosts an {@link OverviewFragment}.
 */
public class OverviewActivity extends BaseMessageActivity {

    public static final int OVERVIEW_ACTIONS_LOADER_ID = 104;
    /**
     * Used by legacy shortcuts.
     */
    private static final String EXTRA_INT_SHOW_TVDBID = "show_tvdbid";
    /**
     * Used by shortcuts.
     */
    private static final String EXTRA_INT_SHOW_TMDBID = "show_tmdbid";
    private static final String EXTRA_LONG_SHOW_ROWID = "show_id";
    private static final String EXTRA_BOOLEAN_DISPLAY_SEASONS = "EXTRA_DISPLAY_SEASONS";

    private long showId;

    /**
     * After opening, switches to overview tab (only if not multi-pane).
     */
    public static Intent intentShowByTmdbId(Context context, int showTmdbId) {
        return new Intent(context, OverviewActivity.class)
                .putExtra(EXTRA_INT_SHOW_TMDBID, showTmdbId);
    }

    /**
     * After opening, switches to overview tab (only if not multi-pane).
     */
    public static Intent intentShow(Context context, long showRowId) {
        return new Intent(context, OverviewActivity.class)
                .putExtra(EXTRA_LONG_SHOW_ROWID, showRowId);
    }

    /**
     * After opening, switches to seasons tab (only if not multi-pane).
     */
    public static Intent intentSeasons(Context context, long showRowId) {
        return intentShow(context, showRowId).putExtra(EXTRA_BOOLEAN_DISPLAY_SEASONS, true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_overview);
        ButterKnife.bind(this);
        setupActionBar();

        SgShow2Helper helper = SgRoomDatabase.getInstance(this).sgShow2Helper();

        showId = getIntent().getLongExtra(EXTRA_LONG_SHOW_ROWID, 0);
        if (showId == 0) {
            // Try to look up by TMDB ID or TVDB ID (used by shortcuts).
            int showTmdbIdOrZero = getIntent().getIntExtra(EXTRA_INT_SHOW_TMDBID, 0);
            int showTvdbIdOrZero = getIntent().getIntExtra(EXTRA_INT_SHOW_TVDBID, 0);
            if (showTmdbIdOrZero > 0) {
                showId = helper.getShowIdByTmdbId(showTmdbIdOrZero);
            } else if (showTvdbIdOrZero > 0) {
                showId = helper.getShowIdByTvdbId(showTvdbIdOrZero);
            }
        }

        if (showId <= 0) {
            finish();
            return;
        }

        setupViews(savedInstanceState);

        updateShowDelayed(showId);
    }

    @Override
    protected void setupActionBar() {
        super.setupActionBar();
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayShowTitleEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
    }

    private void setupViews(Bundle savedInstanceState) {
        // look if we are on a multi-pane or single-pane layout...
        View pagerView = findViewById(R.id.pagerOverview);
        if (pagerView != null && pagerView.getVisibility() == View.VISIBLE) {
            // ...single pane layout with view pager

            // clear up left-over fragments from multi-pane layout
            findAndRemoveFragment(R.id.fragment_overview);
            findAndRemoveFragment(R.id.fragment_seasons);

            setupViewPager(pagerView);
        } else {
            // ...multi-pane overview and seasons fragment

            // clear up left-over fragments from single-pane layout
            boolean isSwitchingLayouts = getActiveFragments().size() != 0;
            for (Fragment fragment : getActiveFragments()) {
                getSupportFragmentManager().beginTransaction().remove(fragment).commit();
            }

            // attach new fragments if there are none or if we just switched
            // layouts
            if (savedInstanceState == null || isSwitchingLayouts) {
                setupPanes();
            }
        }
    }

    private void setupPanes() {
        Fragment showsFragment = new ShowFragment(showId);
        FragmentTransaction ft1 = getSupportFragmentManager().beginTransaction();
        ft1.setCustomAnimations(R.anim.fade_in, R.anim.fade_out);
        ft1.replace(R.id.fragment_show, showsFragment);
        ft1.commit();

        Fragment overviewFragment = new OverviewFragment(showId);
        FragmentTransaction ft2 = getSupportFragmentManager().beginTransaction();
        ft2.setCustomAnimations(R.anim.fade_in, R.anim.fade_out);
        ft2.replace(R.id.fragment_overview, overviewFragment);
        ft2.commit();

        Fragment seasonsFragment = new SeasonsFragment(showId);
        FragmentTransaction ft3 = getSupportFragmentManager().beginTransaction();
        ft3.setCustomAnimations(R.anim.fade_in, R.anim.fade_out);
        ft3.replace(R.id.fragment_seasons, seasonsFragment);
        ft3.commit();
    }

    private void setupViewPager(View pagerView) {
        ViewPager pager = (ViewPager) pagerView;

        // setup tab strip
        TabStripAdapter tabsAdapter = new TabStripAdapter(getSupportFragmentManager(), this, pager,
                findViewById(R.id.tabsOverview));
        tabsAdapter.addTab(R.string.show_details, ShowFragment.class, ShowFragment.buildArgs(showId));

        tabsAdapter.addTab(R.string.description_overview, OverviewFragment.class, OverviewFragment.buildArgs(showId));

        tabsAdapter.addTab(R.string.episodes, SeasonsFragment.class, SeasonsFragment.buildArgs(showId));
        tabsAdapter.notifyTabsChanged();

        // select overview to be shown initially
        boolean displaySeasons = getIntent().getBooleanExtra(EXTRA_BOOLEAN_DISPLAY_SEASONS, false);
        pager.setCurrentItem(displaySeasons ? 2 /* seasons */ : 1 /* overview */);
    }

    private void findAndRemoveFragment(int fragmentId) {
        Fragment overviewFragment = getSupportFragmentManager().findFragmentById(fragmentId);
        if (overviewFragment != null) {
            getSupportFragmentManager().beginTransaction().remove(overviewFragment).commit();
        }
    }

    List<WeakReference<Fragment>> fragments = new ArrayList<>();

    @Override
    public void onAttachFragment(Fragment fragment) {
        /*
         * View pager fragments have tags set by the pager, we can use this to
         * only add refs to those then, making them available to get removed if
         * we switch to a non-pager layout.
         */
        if (fragment.getTag() != null) {
            fragments.add(new WeakReference<>(fragment));
        }
    }

    public ArrayList<Fragment> getActiveFragments() {
        ArrayList<Fragment> ret = new ArrayList<>();
        for (WeakReference<Fragment> ref : fragments) {
            Fragment f = ref.get();
            if (f != null) {
                if (f.isAdded()) {
                    ret.add(f);
                }
            }
        }
        return ret;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.overview_activity_menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        if (itemId == R.id.menu_overview_search) {
            launchSearch();
            return true;
        }
        if (itemId == R.id.menu_show_manage_lists) {
            ManageListsDialogFragment.show(getSupportFragmentManager(), showId);
            return true;
        }
        if (itemId == R.id.menu_overview_remove_show) {
            RemoveShowDialogFragment.show(showId, getSupportFragmentManager(), this);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onEventMainThread(ShowTools2.OnRemovingShowEvent event) {
        if (event.getShowId() == showId) {
            finish(); // finish this activity if the show it displays is about to get removed
        }
    }

    private void launchSearch() {
        // refine search with the show's title
        String titleOrNull = SgRoomDatabase.getInstance(this).sgShow2Helper().getShowTitle(showId);
        if (titleOrNull != null) {
            Bundle appSearchData = new Bundle();
            appSearchData.putString(EpisodeSearchFragment.ARG_SHOW_TITLE, titleOrNull);

            Intent intent = new Intent(this, SearchActivity.class);
            intent.putExtra(SearchManager.APP_DATA, appSearchData);
            intent.setAction(Intent.ACTION_SEARCH);
            startActivity(intent);
        }
    }

    @Override
    protected View getSnackbarParentView() {
        if (getResources().getBoolean(R.bool.isOverviewSinglePane)) {
            return findViewById(R.id.coordinatorLayoutOverview);
        } else {
            return super.getSnackbarParentView();
        }
    }
}
